"""Add registrar column

Revision ID: 912bbcb77a6c
Revises: 6f4729deaf03
Create Date: 2021-12-04 23:52:11.861732

"""
import sqlalchemy as sa
from alembic import op

# pylint: skip-file

# revision identifiers, used by Alembic.
revision = "912bbcb77a6c"
down_revision = "6f4729deaf03"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "pyfunceble_whois_record", sa.Column("registrar", sa.Text(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("pyfunceble_whois_record", "registrar")
    # ### end Alembic commands ###
