"""Add continue table.

Revision ID: 3a4c55a9320d
Revises: 3d6f4a33cdb2
Create Date: 2020-12-13 22:52:56.968513

"""
import sqlalchemy as sa
from alembic import op

# pylint: skip-file

# revision identifiers, used by Alembic.
revision = "3a4c55a9320d"
down_revision = "3d6f4a33cdb2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "pyfunceble_continue",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("modified_at", sa.DateTime(), nullable=True),
        sa.Column("idna_subject", sa.Text(), nullable=False),
        sa.Column("checker_type", sa.String(length=50), nullable=False),
        sa.Column("destination", sa.Text(), nullable=False),
        sa.Column("source", sa.Text(), nullable=False),
        sa.Column("tested_at", sa.DateTime(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("pyfunceble_continue")
    # ### end Alembic commands ###
